@@ -123,7 +123,7 @@ axel_t *axel_new( conf_t *conf, int count, void *url )
 	if( ( axel->size = axel->conn[0].size ) != INT_MAX )
 	{
 		if( axel->conf->verbose > 0 )
-			axel_message( axel, _("File size: %i bytes"), axel->size );
+			axel_message( axel, _("File size: %lld bytes"), axel->size );
 	}
 	
 	/* Wildcards in URL --> Get complete filename			*/
@@ -137,6 +137,7 @@ axel_t *axel_new( conf_t *conf, int count, void *url )
 int axel_open( axel_t *axel )
 {
 	int i, fd;
+	long long int j;
 	
 	if( axel->conf->verbose > 0 )
 		axel_message( axel, _("Opening output file %s"), axel->filename );
@@ -167,7 +168,7 @@ int axel_open( axel_t *axel )
 		for( i = 0; i < axel->conf->num_connections; i ++ )
 			read( fd, &axel->conn[i].currentbyte, sizeof( axel->conn[i].currentbyte ) );
 
-		axel_message( axel, _("State file found: %i bytes downloaded, %i to go."),
+		axel_message( axel, _("State file found: %lld bytes downloaded, %lld to go."),
 			axel->bytes_done, axel->size - axel->bytes_done );
 		
 		close( fd );
@@ -201,11 +202,11 @@ int axel_open( axel_t *axel )
 			axel_message( axel, _("Crappy filesystem/OS.. Working around. :-(") );
 			lseek( axel->outfd, 0, SEEK_SET );
 			memset( buffer, 0, axel->conf->buffer_size );
-			i = axel->size;
-			while( i > 0 )
+			j = axel->size;
+			while( j > 0 )
 			{
-				write( axel->outfd, buffer, min( i, axel->conf->buffer_size ) );
-				i -= axel->conf->buffer_size;
+				write( axel->outfd, buffer, min( j, axel->conf->buffer_size ) );
+				j -= axel->conf->buffer_size;
 			}
 		}
 	}
@@ -260,7 +261,8 @@ void axel_start( axel_t *axel )
 void axel_do( axel_t *axel )
 {
 	fd_set fds[1];
-	int hifd, i, j, size;
+	int hifd, i;
+	long long int remaining,size;
 	struct timeval timeval[1];
 	
 	/* Create statefile if necessary				*/
@@ -338,9 +340,9 @@ void axel_do( axel_t *axel )
 			conn_disconnect( &axel->conn[i] );
 			continue;
 		}
-		/* j == Bytes to go					*/
-		j = axel->conn[i].lastbyte - axel->conn[i].currentbyte + 1;
-		if( j < size )
+		/* remaining == Bytes to go					*/
+		remaining = axel->conn[i].lastbyte - axel->conn[i].currentbyte + 1;
+		if( remaining < size )
 		{
 			if( axel->conf->verbose )
 			{
@@ -348,7 +350,7 @@ void axel_do( axel_t *axel )
 			}
 			axel->conn[i].enabled = 0;
 			conn_disconnect( &axel->conn[i] );
-			size = j;
+			size = remaining;
 			/* Don't terminate, still stuff to write!	*/
 		}
 		/* This should always succeed..				*/
@@ -384,7 +386,10 @@ conn_check:
 		if( !axel->conn[i].enabled && axel->conn[i].currentbyte < axel->conn[i].lastbyte )
 		{
 			if( axel->conn[i].state == 0 )
-			{
+			{	
+				// Wait for termination of this thread
+				pthread_join(*(axel->conn[i].setup_thread), NULL);
+				
 				conn_set( &axel->conn[i], axel->url->text );
 				axel->url = axel->url->next;
 				/* axel->conn[i].local_if = axel->conf->interfaces->text;
@@ -571,13 +576,13 @@ static void axel_divide( axel_t *axel )
 	for( i = 1; i < axel->conf->num_connections; i ++ )
 	{
 #ifdef DEBUG
-		printf( "Downloading %i-%i using conn. %i\n", axel->conn[i-1].currentbyte, axel->conn[i-1].lastbyte, i - 1 );
+		printf( "Downloading %lld-%lld using conn. %i\n", axel->conn[i-1].currentbyte, axel->conn[i-1].lastbyte, i - 1 );
 #endif
 		axel->conn[i].currentbyte = axel->conn[i-1].lastbyte + 1;
 		axel->conn[i].lastbyte = axel->conn[i].currentbyte + axel->size / axel->conf->num_connections;
 	}
 	axel->conn[axel->conf->num_connections-1].lastbyte = axel->size - 1;
 #ifdef DEBUG
-	printf( "Downloading %i-%i using conn. %i\n", axel->conn[i-1].currentbyte, axel->conn[i-1].lastbyte, i - 1 );
+	printf( "Downloading %lld-%lld using conn. %i\n", axel->conn[i-1].currentbyte, axel->conn[i-1].lastbyte, i - 1 );
 #endif
 }
