@@ -65,9 +65,10 @@
 
 /* Compiled-in settings							*/
 #define MAX_STRING		1024
+#define MAX_ADD_HEADERS	10
 #define MAX_REDIR		5
-#define AXEL_VERSION_STRING	"1.1"
-#define USER_AGENT		"Axel " AXEL_VERSION_STRING " (" ARCH ")"
+#define AXEL_VERSION_STRING	"2.3"
+#define DEFAULT_USER_AGENT	"Axel " AXEL_VERSION_STRING " (" ARCH ")"
 
 typedef struct
 {
@@ -95,7 +96,7 @@ typedef struct
 	char filename[MAX_STRING];
 	double start_time;
 	int next_state, finish_time;
-	int bytes_done, start_byte, size;
+	long long bytes_done, start_byte, size;
 	int bytes_per_second;
 	int delay_time;
 	int outfd;
