@@ -103,15 +103,14 @@ void http_get( http_t *conn, char *lurl )
 		http_addheader( conn, "GET %s HTTP/1.0", lurl );
 		http_addheader( conn, "Host: %s", conn->host );
 	}
-	http_addheader( conn, "User-Agent: %s", USER_AGENT );
 	if( *conn->auth )
 		http_addheader( conn, "Authorization: Basic %s", conn->auth );
 	if( conn->firstbyte )
 	{
 		if( conn->lastbyte )
-			http_addheader( conn, "Range: bytes=%i-%i", conn->firstbyte, conn->lastbyte );
+			http_addheader( conn, "Range: bytes=%lld-%lld", conn->firstbyte, conn->lastbyte );
 		else
-			http_addheader( conn, "Range: bytes=%i-", conn->firstbyte );
+			http_addheader( conn, "Range: bytes=%lld-", conn->firstbyte );
 	}
 }
 
@@ -196,15 +195,15 @@ char *http_header( http_t *conn, char *header )
 	return( NULL );
 }
 
-int http_size( http_t *conn )
+long long int http_size( http_t *conn )
 {
 	char *i;
-	int j;
+	long long int j;
 	
 	if( ( i = http_header( conn, "Content-Length:" ) ) == NULL )
 		return( -2 );
 	
-	sscanf( i, "%i", &j );
+	sscanf( i, "%lld", &j );
 	return( j );
 }
 
@@ -236,9 +235,19 @@ void http_encode( char *s )
 	
 	for( i = j = 0; s[i]; i ++, j ++ )
 	{
+		/* Fix buffer overflow */
+		if (j >= MAX_STRING - 1) {
+			break;
+		}
+		
 		t[j] = s[i];
 		if( s[i] == ' ' )
 		{
+			/* Fix buffer overflow */
+			if (j >= MAX_STRING - 3) {
+				break;
+			}
+			
 			strcpy( t + j, "%20" );
 			j += 2;
 		}
